@@ -1,18 +1,18 @@
 import collections.abc
+import json
+import operator
+import re
 import sys
 import typing
-from collections.abc import Sequence
-from datetime import datetime, timedelta
-from enum import Enum
-from functools import partial
-from inspect import isclass
+from collections.abc import Callable, Iterable, Sequence
+from datetime import date, datetime, timedelta
+from enum import Enum, Flag
+from functools import partial, reduce
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
-    Iterable,
     Literal,
-    Optional,
+    TypeVar,
     Union,
     get_args,
     get_origin,
@@ -23,10 +23,10 @@ if sys.version_info >= (3, 12):
 else:
     TypeAliasType = None
 
-from cyclopts.annotations import is_annotated, is_nonetype, is_union, resolve
+from cyclopts.annotations import is_annotated, is_enum_flag, is_nonetype, is_union, resolve
 from cyclopts.exceptions import CoercionError, ValidationError
-from cyclopts.field_info import get_field_infos
-from cyclopts.utils import UNSET, default_name_transform, grouper, is_builtin
+from cyclopts.field_info import FieldInfo, get_field_infos
+from cyclopts.utils import UNSET, default_name_transform, grouper, is_builtin, is_class_and_subclass
 
 if sys.version_info >= (3, 12):  # pragma: no cover
     from typing import TypeAliasType
@@ -37,6 +37,10 @@ if TYPE_CHECKING:
     from cyclopts.argument import Token
 
 
+T = TypeVar("T")
+E = TypeVar("E", bound=Enum)
+F = TypeVar("F", bound=Flag)
+
 _implicit_iterable_type_mapping: dict[type, type] = {
     Iterable: list[str],
     typing.Sequence: list[str],
@@ -79,9 +83,13 @@ def _int(s: str) -> int:
         return int(s, 8)
     elif s.startswith("0b"):
         return int(s, 2)
-    else:
+    elif "." in s:
         # Casting to a float first allows for things like "30.0"
+        # We handle this conditionally because very large integers can lose
+        # meaningful precision when cast to a float.
         return int(round(float(s)))
+    else:
+        return int(s)
 
 
 def _bytes(s: str) -> bytes:
@@ -92,6 +100,16 @@ def _bytearray(s: str) -> bytearray:
     return bytearray(_bytes(s))
 
 
+def _date(s: str) -> date:
+    """Parse a date string.
+
+    Returns
+    -------
+    datetime.date
+    """
+    return date.fromisoformat(s)
+
+
 def _datetime(s: str) -> datetime:
     """Parse a datetime string.
 
@@ -99,29 +117,17 @@ def _datetime(s: str) -> datetime:
     -------
     datetime.datetime
     """
-    formats = [
-        # ISO 8601 formats (unambiguous internationally)
-        "%Y-%m-%d",  # 1956-01-31
-        "%Y-%m-%dT%H:%M:%S",  # 1956-01-31T10:00:00
-        "%Y-%m-%d %H:%M:%S",  # 1956-01-31 10:00:00
-        "%Y-%m-%dT%H:%M:%S%z",  # 1956-01-31T10:00:00+0000
-        "%Y-%m-%dT%H:%M:%S.%f",  # 1956-01-31T10:00:00.123456
-        "%Y-%m-%dT%H:%M:%S.%f%z",  # 1956-01-31T10:00:00.123456+0000
-    ]
-
-    for fmt in formats:
-        try:
-            return datetime.strptime(s, fmt)
-        except ValueError:
-            continue
-
-    raise ValueError
+    try:
+        return datetime.fromisoformat(s)
+    except ValueError:
+        # Fallback for space-separated format (not ISO 8601 compliant)
+        # Python 3.11+ fromisoformat() accepts spaces, but 3.10 doesn't
+        # Convert space to 'T' to make it ISO-compliant
+        return datetime.fromisoformat(s.strip().replace(" ", "T", 1))
 
 
 def _timedelta(s: str) -> timedelta:
     """Parse a timedelta string."""
-    import re
-
     negative = False
     if s.startswith("-"):
         negative = True
@@ -157,12 +163,69 @@ def _timedelta(s: str) -> timedelta:
     return timedelta(seconds=seconds)
 
 
+def get_enum_member(
+    type_: type[E],
+    token: Union["Token", str],
+    name_transform: Callable[[str], str],
+) -> E:
+    """Match a token's value to an enum's member.
+
+    Applies ``name_transform`` to both the value and the member.
+    """
+    from cyclopts.argument import Token
+
+    is_token = isinstance(token, Token)
+    value = token.value if is_token else token
+    value_transformed = name_transform(value)
+    for name, member in type_.__members__.items():
+        if name_transform(name) == value_transformed:
+            return member
+    raise CoercionError(
+        token=token if is_token else None,
+        target_type=type_,
+    )
+
+
+def convert_enum_flag(
+    enum_type: type[F],
+    tokens: Iterable[str] | Iterable["Token"],
+    name_transform: Callable[[str], str],
+) -> F:
+    """Convert tokens to a Flag enum value.
+
+    Parameters
+    ----------
+    enum_type : type[F]
+        The Flag enum type to convert to.
+    tokens : Iterable[str] | Iterable[Token]
+        The tokens to convert. Can be member names or :class:`Token` objects.
+    name_transform : Callable[[str], str] | None
+        Function to transform names for comparison.
+
+    Returns
+    -------
+    F
+        The combined flag value.
+
+    Raises
+    ------
+    CoercionError
+        If a token is not a valid flag member.
+    """
+    return reduce(
+        operator.or_,
+        (get_enum_member(enum_type, token, name_transform) for token in tokens),
+        enum_type(0),
+    )
+
+
 # For types that need more logic than just invoking their type
 _converters: dict[Any, Callable] = {
     bool: _bool,
     int: _int,
     bytes: _bytes,
     bytearray: _bytearray,
+    date: _date,
     datetime: _datetime,
     timedelta: _timedelta,
 }
@@ -171,7 +234,7 @@ _converters: dict[Any, Callable] = {
 def _convert_tuple(
     type_: type[Any],
     *tokens: "Token",
-    converter: Optional[Callable[[type, str], Any]],
+    converter: Callable[[type, str], Any] | None,
     name_transform: Callable[[str], str],
 ) -> tuple:
     convert = partial(_convert, converter=converter, name_transform=name_transform)
@@ -207,15 +270,226 @@ def _convert_tuple(
         it = iter(tokens)
         batched = [[next(it) for _ in range(size)] for size in args_per_convert]
         batched = [elem[0] if len(elem) == 1 else elem for elem in batched]
-        out = tuple(convert(inner_type, arg) for inner_type, arg in zip(inner_types, batched))
+        out = tuple(convert(inner_type, arg) for inner_type, arg in zip(inner_types, batched, strict=False))
     return out
 
 
+def _convert_json(
+    type_: Any,
+    data: dict,
+    field_infos: dict,
+    converter: Callable | None,
+    name_transform: Callable[[str], str],
+):
+    """Convert JSON dict to dataclass with proper type conversion for fields.
+
+    Parameters
+    ----------
+    type_ : Type
+        The dataclass type to create.
+    data : dict
+        The JSON dictionary containing field values.
+    field_infos : dict
+        Field information from the dataclass.
+    converter : Callable | None
+        Optional converter function.
+    name_transform : Callable[[str], str]
+        Function to transform field names.
+
+    Returns
+    -------
+    Instance of type_ with properly converted field values.
+    """
+    from cyclopts.token import Token
+
+    converted_data = {}
+    for field_name, field_info in field_infos.items():
+        if field_name in data:
+            value = data[field_name]
+            # Convert the value to the proper type
+            if value is not None and not is_class_and_subclass(field_info.hint, str):
+                # Create a token for the value and convert it
+                token = Token(value=json.dumps(value) if isinstance(value, dict | list) else str(value))
+                # Always attempt conversion, let errors propagate for consistency
+                converted_value = convert(field_info.hint, [token], converter, name_transform)
+            else:
+                converted_value = value
+            converted_data[field_name] = converted_value
+
+    # Create the dataclass with converted values
+    return type_(**converted_data)
+
+
+def _create_json_decode_error_message(
+    token: "Token",
+    type_: Any,
+    error: json.JSONDecodeError,
+) -> str:
+    """Create a helpful error message for JSON decode errors.
+
+    Parameters
+    ----------
+    token : Token
+        The token containing the invalid JSON.
+    type_ : Type
+        The target type we were trying to convert to.
+    error : json.JSONDecodeError
+        The JSON decode error that occurred.
+
+    Returns
+    -------
+    str
+        A formatted error message with context and hints.
+    """
+    value_str = token.value.strip()
+
+    # Try to provide context around the error
+    error_pos = error.pos if hasattr(error, "pos") else error.colno - 1 if hasattr(error, "colno") else 0
+
+    # Create a snippet showing the error location
+    snippet_start = max(0, error_pos - 20)
+    snippet_end = min(len(value_str), error_pos + 20)
+    snippet = value_str[snippet_start:snippet_end]
+
+    # Add markers if we truncated
+    if snippet_start > 0:
+        snippet = "..." + snippet
+    if snippet_end < len(value_str):
+        snippet = snippet + "..."
+
+    # Calculate where the error marker should point
+    marker_pos = error_pos - snippet_start
+    if snippet_start > 0:
+        marker_pos += 3  # Account for "..."
+
+    # Common error patterns with helpful hints
+    hint = ""
+    if re.search(r"\bTrue\b", value_str):
+        hint = "\n    Hint: Use lowercase 'true' instead of Python's True"
+    elif re.search(r"\bFalse\b", value_str):
+        hint = "\n    Hint: Use lowercase 'false' instead of Python's False"
+    elif re.search(r"\bNone\b", value_str):
+        hint = "\n    Hint: Use 'null' instead of Python's None"
+    elif "'" in value_str:
+        hint = "\n    Hint: JSON requires double quotes, not single quotes"
+
+    return f"Invalid JSON for {type_.__name__}:\n    {snippet}\n    {' ' * marker_pos}^ {error.msg}{hint}"
+
+
+def instantiate_from_dict(type_: type[T], data: dict[str, Any]) -> T:
+    """Instantiate a type with proper handling of parameter kinds.
+
+    Respects POSITIONAL_ONLY, KEYWORD_ONLY, and POSITIONAL_OR_KEYWORD parameter kinds
+    when constructing the object.
+
+    This function is necessary because `inspect.signature().bind(**data)` has the same
+    limitation we're solving: it cannot accept positional-only parameters as keyword
+    arguments. For example, `def __init__(self, a, /, b)` requires `a` to be passed
+    positionally, but when we have a dict `{"a": 1, "b": 2}`, we need to transform
+    this into the call `type_(1, b=2)`.
+
+    Parameters
+    ----------
+    type_ : type[T]
+        The type to instantiate.
+    data : dict[str, Any]
+        Dictionary mapping field names to values.
+
+    Returns
+    -------
+    T
+        Instance of type_ constructed from data.
+    """
+    field_infos = get_field_infos(type_)
+    if not field_infos:
+        return type_(**data)
+
+    pos_args = []
+    kwargs = {}
+
+    for field_name, value in data.items():
+        field_info = field_infos.get(field_name)
+        if field_info and field_info.kind == FieldInfo.POSITIONAL_ONLY:
+            pos_args.append((field_name, value))
+        else:
+            kwargs[field_name] = value
+
+    # Sort positional args by their order in field_infos
+    field_names_order = list(field_infos.keys())
+    pos_args.sort(key=lambda x: field_names_order.index(x[0]))
+
+    return type_(*(v for _, v in pos_args), **kwargs)
+
+
+def _convert_structured_type(
+    type_: type[T],
+    token: Sequence["Token"],
+    field_infos: dict[str, "FieldInfo"],
+    convert: Callable,
+) -> T:
+    """Convert tokens to a structured type with proper positional/keyword argument handling.
+
+    Respects the parameter kind of each field:
+    - POSITIONAL_ONLY: passed as positional argument
+    - KEYWORD_ONLY or POSITIONAL_OR_KEYWORD: passed as keyword argument
+
+    This correctly handles types with keyword-only fields (e.g., dataclasses with kw_only=True).
+
+    Parameters
+    ----------
+    type_ : type[T]
+        The target structured type to instantiate.
+    token : Sequence[Token]
+        The tokens to convert.
+    field_infos : dict[str, FieldInfo]
+        Field information for the structured type.
+    convert : Callable
+        Conversion function for nested types.
+
+    Returns
+    -------
+    T
+        Instance of type_ constructed from the tokens.
+    """
+    i = 0
+    data = {}
+    hint = type_
+
+    for field_name, field_info in field_infos.items():
+        hint = field_info.hint
+
+        # Convert the token(s) for this field
+        if is_class_and_subclass(hint, str):  # Avoids infinite recursion
+            value = token[i].value
+            i += 1
+            should_break = False
+        else:
+            tokens_per_element, consume_all = token_count(hint)
+            if tokens_per_element == 1:
+                value = convert(hint, token[i])
+                i += 1
+            else:
+                value = convert(hint, token[i : i + tokens_per_element])
+                i += tokens_per_element
+            should_break = consume_all
+
+        data[field_name] = value
+
+        # Handle consume_all or end of tokens
+        if should_break:
+            break
+        if i == len(token):
+            break
+
+    assert i == len(token)
+    return instantiate_from_dict(type_, data)
+
+
 def _convert(
     type_,
     token: Union["Token", Sequence["Token"]],
     *,
-    converter: Optional[Callable[[Any, str], Any]],
+    converter: Callable[[Any, str], Any] | None,
     name_transform: Callable[[str], str],
 ):
     """Inner recursive conversion function for public ``convert``.
@@ -260,7 +534,7 @@ def _convert(
         out = convert(_implicit_iterable_type_mapping[type_], token)
     elif origin_type in (collections.abc.Iterable, collections.abc.Sequence):
         assert len(inner_types) == 1
-        out = convert(list[inner_types[0]], token)  # pyright: ignore[reportGeneralTypeIssues]
+        out = convert(list[inner_types[0]], token)
     elif TypeAliasType is not None and isinstance(type_, TypeAliasType):
         out = convert(type_.__value__, token)
     elif is_union(origin_type):
@@ -305,23 +579,30 @@ def _convert(
         count, _ = token_count(inner_types[0])
         if not isinstance(token, Sequence):
             raise ValueError
-        if count > 1:
-            gen = zip(*[iter(token)] * count)
+
+        # Check if tokens are JSON strings
+        inner_type = inner_types[0]
+        if (
+            count > 1
+            and any(isinstance(t, Token) and t.value.strip().startswith("{") for t in token)
+            and inner_type is not str
+        ):
+            # Each token is a complete JSON representation of the dataclass
+            gen = token
+        elif count > 1:
+            gen = zip(*[iter(token)] * count, strict=False)
         else:
             gen = token
-        out = origin_type(convert(inner_types[0], e) for e in gen)  # pyright: ignore[reportOptionalCall]
-    elif isclass(type_) and issubclass(type_, Enum):
+        out = origin_type(convert(inner_types[0], e) for e in gen)
+    elif is_class_and_subclass(type_, Flag):
+        # TODO: this might never execute since enum.Flag is now handled in ``convert``.
+        out = convert_enum_flag(type_, token if isinstance(token, Sequence) else [token], name_transform)
+    elif is_class_and_subclass(type_, Enum):
         if isinstance(token, Sequence):
             raise ValueError
 
         if converter is None:
-            element_transformed = name_transform(token.value)
-            for name, member in type_.__members__.items():
-                if name_transform(name) == element_transformed:
-                    out = member
-                    break
-            else:
-                raise CoercionError(token=token, target_type=type_)
+            out = get_enum_member(type_, token, name_transform)
         else:
             out = converter(type_, token.value)
     else:
@@ -335,7 +616,7 @@ def _convert(
                 if token.implicit_value is not UNSET:
                     out = token.implicit_value
                 elif converter is None:
-                    out = _converters.get(type_, type_)(token.value)
+                    out = _converters.get(type_, type_)(token.value)  # pyright: ignore[reportOptionalCall]
                 elif converter_needs_token:
                     out = converter(type_, token)  # pyright: ignore[reportArgumentType]
                 else:
@@ -350,30 +631,29 @@ def _convert(
                 raise CoercionError(token=token, target_type=type_) from None
         else:
             # Convert it into a user-supplied class.
-            if not isinstance(token, Sequence):
-                token = [token]
-            i = 0
-            pos_values = []
-            hint = type_
-            for field_info in field_infos.values():
-                hint = field_info.hint
-                if isclass(hint) and issubclass(hint, str):  # Avoids infinite recursion
-                    pos_values.append(token[i].value)
-                    i += 1
-                else:
-                    tokens_per_element, consume_all = token_count(hint)
-                    if tokens_per_element == 1:
-                        pos_values.append(convert(hint, token[i]))
-                        i += 1
-                    else:
-                        pos_values.append(convert(hint, token[i : i + tokens_per_element]))
-                        i += tokens_per_element
-                    if consume_all:
-                        break
-                if i == len(token):
-                    break
-            assert i == len(token)
-            out = type_(*pos_values)
+            # First check if we have a single token that's a JSON string
+            if isinstance(token, Token) and token.value.strip().startswith("{") and type_ is not str:
+                try:
+                    data = json.loads(token.value)
+                    if not isinstance(data, dict):
+                        # JSON was valid but didn't produce a dict (e.g., it was an array or scalar)
+                        raise TypeError  # noqa: TRY301
+                    # Convert dict to dataclass with proper type conversion
+                    out = _convert_json(type_, data, field_infos, converter, name_transform)
+                except json.JSONDecodeError as e:
+                    # Create helpful error message for invalid JSON
+                    msg = _create_json_decode_error_message(token, type_, e)
+                    raise CoercionError(msg=msg, token=token, target_type=type_) from e
+                except TypeError:
+                    # Fall back to positional argument parsing
+                    if not isinstance(token, Sequence):
+                        token = [token]
+                    out = _convert_structured_type(type_, token, field_infos, convert)
+            else:
+                # Standard positional argument parsing
+                if not isinstance(token, Sequence):
+                    token = [token]
+                out = _convert_structured_type(type_, token, field_infos, convert)
 
     if cparam:
         # An inner type may have an independent Parameter annotation;
@@ -391,9 +671,9 @@ def _convert(
 
 def convert(
     type_: Any,
-    tokens: Union[Sequence[str], Sequence["Token"], NestedCliArgs],
-    converter: Optional[Callable[[type, str], Any]] = None,
-    name_transform: Optional[Callable[[str], str]] = None,
+    tokens: Sequence[str] | Sequence["Token"] | NestedCliArgs,
+    converter: Callable[[type, str], Any] | None = None,
+    name_transform: Callable[[str], str] | None = None,
 ):
     """Coerce variables into a specified type.
 
@@ -484,9 +764,13 @@ def convert(
         dict_converted = {
             k: convert(value_type, v, converter=converter, name_transform=name_transform) for k, v in tokens.items()
         }
-        return _converters.get(maybe_origin_type, maybe_origin_type)(**dict_converted)  # pyright: ignore
+        return _converters.get(maybe_origin_type, maybe_origin_type)(**dict_converted)
     elif isinstance(tokens, dict):
         raise ValueError(f"Dictionary of tokens provided for unknown {type_!r}.")  # Programming error
+    elif is_enum_flag(maybe_origin_type):
+        # Unlike other types that can accept multiple tokens, the result is not a sequence, it's a single
+        # enum.Flag object.
+        return convert_enum_flag(maybe_origin_type, tokens, name_transform)
     else:
         if len(tokens) == 1:
             return convert_priv(type_, tokens[0])  # pyright: ignore
@@ -528,6 +812,8 @@ def token_count(type_: Any) -> tuple[int, bool]:
         return 0, False
     elif type_ in ITERABLE_TYPES or (origin_type in ITERABLE_TYPES and len(get_args(type_)) == 0):
         return 1, True
+    elif is_enum_flag(type_):
+        return 1, True
     elif (origin_type in ITERABLE_TYPES or origin_type is collections.abc.Iterable) and len(get_args(type_)):
         return token_count(get_args(type_)[0])[0], True
     elif TypeAliasType is not None and isinstance(type_, TypeAliasType):
@@ -558,7 +844,7 @@ def token_count(type_: Any) -> tuple[int, bool]:
             count += elem_count
             consume_all |= elem_consume_all
 
-        # classes like ``Enum`` can slip through here with a 0 count.
+        # classes like ``enum.Enum`` can slip through here with a 0 count.
         if not count:
             return 1, False
 
