@@ -1,6 +1,8 @@
+import json
+from collections.abc import Sequence
 from functools import partial
 from pathlib import Path
-from typing import TYPE_CHECKING, Annotated, Any, Sequence
+from typing import TYPE_CHECKING, Annotated, Any
 
 from cyclopts import validators
 from cyclopts.parameter import Parameter
@@ -260,8 +262,6 @@ HexUInt64 = Annotated[UInt64, Parameter(show_default=partial(_hex_formatter, dig
 # Json #
 ########
 def _json_converter(type_, tokens: Sequence["Token"]):
-    import json
-
     assert len(tokens) == 1
     out = json.loads(tokens[0].value)
     return out
